import { __classPrivateFieldGet, __classPrivateFieldSet } from 'tslib';
import assert from 'assert';
import { createSocket } from 'dgram';
import { Socket, isIPv6, isIP } from 'net';
import { DefaultSettings } from '../constants/DefaultSettings.js';

var _TinySocket_instances, _TinySocket_client, _TinySocket_connected, _TinySocket_isConnected, _TinySocket_assertConnected, _TinySocket_assertDisconnected;
const MacAddressMatcher = /^([0-9a-f]{2}:){5}([0-9a-f]{2})$/i;
const MacAddressBlockSeparator = ":";
const MacAddressBlocksCount = 6;
const WolMacAddressCount = 16;
const WolSyncByte = 255;
const WolSyncCount = 6;
function assertSettings(settings) {
  assert(typeof settings === "object" && settings !== null, "settings must be an object");
  const { networkPort, networkTimeout, networkWolAddress, networkWolPort } = settings;
  assert(typeof networkPort === "number" && networkPort > 0, "settings.networkPort must be a number greater than 0");
  assert(typeof networkTimeout === "number" && networkTimeout > 0, "settings.networkTimeout must be a number greater than 0");
  assert(typeof networkWolAddress === "string" && networkWolAddress.length > 0, "settings.networkWolAddress must be a string with length greater than 0");
  assert(isIP(networkWolAddress), "settings.networkWolAddress must be a valid IPv4 or IPv6");
  assert(typeof networkWolPort === "number" && networkWolPort > 0, "settings.networkWolPort must be a number greater than 0");
}
class MaxRetriesError extends Error {
}
class TimeoutError extends Error {
}
class TinySocket {
  constructor(host, macAddress, settings = DefaultSettings) {
    _TinySocket_instances.add(this);
    this.host = host;
    this.macAddress = macAddress;
    this.settings = settings;
    _TinySocket_client.set(this, new Socket());
    _TinySocket_connected.set(this, false);
    assertSettings(settings);
    assert(macAddress === null || MacAddressMatcher.test(macAddress), "invalid mac address");
  }
  get connected() {
    return __classPrivateFieldGet(this, _TinySocket_instances, "m", _TinySocket_isConnected).call(this);
  }
  wrap(method) {
    return new Promise((resolve, reject) => {
      const handleTimeout = () => {
        __classPrivateFieldSet(this, _TinySocket_connected, false, "f");
        __classPrivateFieldGet(this, _TinySocket_client, "f").end();
        reject(new TimeoutError());
      };
      const cleanup = () => {
        __classPrivateFieldGet(this, _TinySocket_client, "f").removeListener("error", reject);
        __classPrivateFieldGet(this, _TinySocket_client, "f").removeListener("timeout", handleTimeout);
      };
      __classPrivateFieldGet(this, _TinySocket_client, "f").once("error", reject);
      __classPrivateFieldGet(this, _TinySocket_client, "f").once("timeout", handleTimeout);
      method((value) => {
        cleanup();
        resolve(value);
      }, (error) => {
        cleanup();
        reject(error);
      });
    });
  }
  async connect(options = {}) {
    __classPrivateFieldGet(this, _TinySocket_instances, "m", _TinySocket_assertDisconnected).call(this);
    const { maxRetries = 0, retryTimeout = 750 } = options;
    if (maxRetries > 0) {
      return new Promise((resolve, reject) => {
        let retries = 0;
        const connect = (error) => {
          __classPrivateFieldGet(this, _TinySocket_client, "f").removeAllListeners();
          __classPrivateFieldGet(this, _TinySocket_client, "f").destroy();
          __classPrivateFieldSet(this, _TinySocket_client, new Socket(), "f");
          if (retries >= maxRetries) {
            const maxRetriesError = new MaxRetriesError(`maximum retries of ${retries} reached`);
            maxRetriesError.cause = error;
            reject(maxRetriesError);
            return;
          }
          __classPrivateFieldGet(this, _TinySocket_client, "f").setTimeout(retryTimeout);
          __classPrivateFieldGet(this, _TinySocket_client, "f").connect(this.settings.networkPort, this.host);
          __classPrivateFieldGet(this, _TinySocket_client, "f").on("error", connect);
          __classPrivateFieldGet(this, _TinySocket_client, "f").on("timeout", connect);
          __classPrivateFieldGet(this, _TinySocket_client, "f").on("connect", connected);
          retries++;
        };
        const connected = () => {
          __classPrivateFieldGet(this, _TinySocket_client, "f").removeAllListeners();
          __classPrivateFieldGet(this, _TinySocket_client, "f").setTimeout(this.settings.networkTimeout);
          __classPrivateFieldSet(this, _TinySocket_connected, true, "f");
          resolve(void 0);
        };
        connect();
      });
    }
    await this.wrap((resolve) => {
      __classPrivateFieldGet(this, _TinySocket_client, "f").setTimeout(this.settings.networkTimeout);
      __classPrivateFieldGet(this, _TinySocket_client, "f").connect(this.settings.networkPort, this.host, () => {
        resolve(void 0);
      });
      __classPrivateFieldSet(this, _TinySocket_connected, true, "f");
    });
  }
  read() {
    __classPrivateFieldGet(this, _TinySocket_instances, "m", _TinySocket_assertConnected).call(this);
    return this.wrap((resolve) => {
      __classPrivateFieldGet(this, _TinySocket_client, "f").once("data", resolve);
    });
  }
  write(data) {
    __classPrivateFieldGet(this, _TinySocket_instances, "m", _TinySocket_assertConnected).call(this);
    return this.wrap((resolve, reject) => {
      __classPrivateFieldGet(this, _TinySocket_client, "f").write(data, (error) => {
        if (error) {
          reject(error);
        } else {
          resolve();
        }
      });
    });
  }
  async sendReceive(data) {
    __classPrivateFieldGet(this, _TinySocket_instances, "m", _TinySocket_assertConnected).call(this);
    await this.write(data);
    return this.read();
  }
  disconnect() {
    if (!__classPrivateFieldGet(this, _TinySocket_instances, "m", _TinySocket_isConnected).call(this)) {
      return Promise.resolve(void 0);
    }
    return this.wrap((resolve) => {
      __classPrivateFieldSet(this, _TinySocket_connected, false, "f");
      __classPrivateFieldGet(this, _TinySocket_client, "f").end(resolve);
    });
  }
  wakeOnLan() {
    if (!this.macAddress) {
      throw new Error("Unable to wake on lan: mac address was not configured");
    }
    const socket = createSocket(isIPv6(this.settings.networkWolAddress) ? "udp6" : "udp4");
    socket.on("error", socket.close);
    socket.on("listening", () => {
      socket.setBroadcast(true);
    });
    const magicPacket = Buffer.alloc(WolSyncCount + MacAddressBlocksCount * WolMacAddressCount, WolSyncByte);
    const macAddressBlocks = this.macAddress.split(MacAddressBlockSeparator).map((string) => parseInt(string, 16));
    for (let i = 0; i < WolMacAddressCount; i++) {
      for (let j = 0; j < macAddressBlocks.length; j++) {
        const index = WolSyncCount + i * MacAddressBlocksCount + j;
        magicPacket[index] = macAddressBlocks[j];
      }
    }
    socket.send(magicPacket, 0, magicPacket.length, this.settings.networkWolPort, this.settings.networkWolAddress, () => {
      socket.close();
    });
  }
}
_TinySocket_client = /* @__PURE__ */ new WeakMap(), _TinySocket_connected = /* @__PURE__ */ new WeakMap(), _TinySocket_instances = /* @__PURE__ */ new WeakSet(), _TinySocket_isConnected = function _TinySocket_isConnected2() {
  return __classPrivateFieldGet(this, _TinySocket_connected, "f") && !__classPrivateFieldGet(this, _TinySocket_client, "f").connecting && !__classPrivateFieldGet(this, _TinySocket_client, "f").destroyed;
}, _TinySocket_assertConnected = function _TinySocket_assertConnected2() {
  assert(__classPrivateFieldGet(this, _TinySocket_instances, "m", _TinySocket_isConnected).call(this), "should be connected");
}, _TinySocket_assertDisconnected = function _TinySocket_assertDisconnected2() {
  assert(!__classPrivateFieldGet(this, _TinySocket_instances, "m", _TinySocket_isConnected).call(this), "should not be connected");
};

export { MaxRetriesError, TimeoutError, TinySocket };
//# sourceMappingURL=TinySocket.js.map
